"""
Dummy Django settings file
"""

SECRET_KEY = "dummy-key"

#: These are the installed apps
INSTALLED_APPS = [
    "django.contrib.auth",
    "django.contrib.contenttypes",
    "dummy_django_app",
]

USE_TZ = False
